@@ -38,7 +38,11 @@
 #include "proj/internal/internal.hpp"
 #include "proj/internal/io_internal.hpp"
 
+#include "proj_json_streaming_writer.hpp"
+
 #include <algorithm>
+#include <cmath>
+#include <limits>
 #include <memory>
 #include <string>
 #include <vector>
@@ -229,6 +233,32 @@ GeographicBoundingBoxNNPtr GeographicBoundingBox::create(double west,
                                                          double south,
                                                          double east,
                                                          double north) {
+    if (std::isnan(west) || std::isnan(south) || std::isnan(east) ||
+        std::isnan(north)) {
+        throw InvalidValueTypeException(
+            "GeographicBoundingBox::create() does not accept NaN values");
+    }
+    if (south > north) {
+        throw InvalidValueTypeException(
+            "GeographicBoundingBox::create() does not accept south > north");
+    }
+    // Avoid creating a degenerate bounding box if reduced to a point or a line
+    if (west == east) {
+        if (west > -180)
+            west =
+                std::nextafter(west, -std::numeric_limits<double>::infinity());
+        if (east < 180)
+            east =
+                std::nextafter(east, std::numeric_limits<double>::infinity());
+    }
+    if (south == north) {
+        if (south > -90)
+            south =
+                std::nextafter(south, -std::numeric_limits<double>::infinity());
+        if (north < 90)
+            north =
+                std::nextafter(north, std::numeric_limits<double>::infinity());
+    }
     return GeographicBoundingBox::nn_make_shared<GeographicBoundingBox>(
         west, south, east, north);
 }
@@ -270,7 +300,7 @@ bool GeographicBoundingBox::contains(const GeographicExtentNNPtr &other) const {
     }
 
     if (W == -180.0 && E == 180.0) {
-        return true;
+        return oW != oE;
     }
 
     if (oW == -180.0 && oE == 180.0) {
@@ -313,31 +343,45 @@ bool GeographicBoundingBox::Private::intersects(const Private &other) const {
     const double oN = other.north_;
     const double oS = other.south_;
 
+    // Check intersection along the latitude axis
     if (N < oS || S > oN) {
         return false;
     }
 
+    // Check world coverage of this bbox, and other bbox overlapping
+    // antimeridian (e.g. oW=175 and oE=-175)
+    // Check oW > oE written for symmetry with the intersection() method.
     if (W == -180.0 && E == 180.0 && oW > oE) {
         return true;
     }
 
+    // Check world coverage of othre bbox, and this bbox overlapping
+    // antimeridian (e.g. W=175 and E=-175)
+    // Check W > E written for symmetry with the intersection() method.
     if (oW == -180.0 && oE == 180.0 && W > E) {
         return true;
     }
 
     // Normal bounding box ?
     if (W <= E) {
-        if (oW < oE) {
+        if (oW <= oE) {
             if (std::max(W, oW) < std::min(E, oE)) {
                 return true;
             }
             return false;
         }
 
+        // Bail out on longitudes not in [-180,180]. We could probably make
+        // some sense of them, but this check at least avoid potential infinite
+        // recursion.
+        if (oW > 180 || oE < -180) {
+            return false;
+        }
+
         return intersects(Private(oW, oS, 180.0, oN)) ||
                intersects(Private(-180.0, oS, oE, oN));
 
-        // No: crossing antimerian
+        // No: crossing antimeridian
     } else {
         if (oW <= oE) {
             return other.intersects(*this);
@@ -386,15 +430,20 @@ GeographicBoundingBox::Private::intersection(const Private &otherExtent) const {
     const double oN = otherExtent.north_;
     const double oS = otherExtent.south_;
 
+    // Check intersection along the latitude axis
     if (N < oS || S > oN) {
         return nullptr;
     }
 
+    // Check world coverage of this bbox, and other bbox overlapping
+    // antimeridian (e.g. oW=175 and oE=-175)
     if (W == -180.0 && E == 180.0 && oW > oE) {
         return internal::make_unique<Private>(oW, std::max(S, oS), oE,
                                               std::min(N, oN));
     }
 
+    // Check world coverage of othre bbox, and this bbox overlapping
+    // antimeridian (e.g. W=175 and E=-175)
     if (oW == -180.0 && oE == 180.0 && W > E) {
         return internal::make_unique<Private>(W, std::max(S, oS), E,
                                               std::min(N, oN));
@@ -402,16 +451,23 @@ GeographicBoundingBox::Private::intersection(const Private &otherExtent) const {
 
     // Normal bounding box ?
     if (W <= E) {
-        if (oW < oE) {
-            auto res = internal::make_unique<Private>(
-                std::max(W, oW), std::max(S, oS), std::min(E, oE),
-                std::min(N, oN));
-            if (res->west_ < res->east_) {
-                return res;
+        if (oW <= oE) {
+            const double resW = std::max(W, oW);
+            const double resE = std::min(E, oE);
+            if (resW < resE) {
+                return internal::make_unique<Private>(resW, std::max(S, oS),
+                                                      resE, std::min(N, oN));
             }
             return nullptr;
         }
 
+        // Bail out on longitudes not in [-180,180]. We could probably make
+        // some sense of them, but this check at least avoid potential infinite
+        // recursion.
+        if (oW > 180 || oE < -180) {
+            return nullptr;
+        }
+
         // Return larger of two parts of the multipolygon
         auto inter1 = intersection(Private(oW, oS, 180.0, oN));
         auto inter2 = intersection(Private(-180.0, oS, oE, oN));
@@ -425,7 +481,7 @@ GeographicBoundingBox::Private::intersection(const Private &otherExtent) const {
             return inter1;
         }
         return inter2;
-        // No: crossing antimerian
+        // No: crossing antimeridian
     } else {
         if (oW <= oE) {
             return otherExtent.intersection(*this);
@@ -775,7 +831,7 @@ bool Extent::_isEquivalentTo(const util::IComparable *other,
 
 /** \brief Returns whether this extent contains the other one.
  *
- * Behaviour only well specified if each sub-extent category as at most
+ * Behavior only well specified if each sub-extent category as at most
  * one element.
  */
 bool Extent::contains(const ExtentNNPtr &other) const {
@@ -800,7 +856,7 @@ bool Extent::contains(const ExtentNNPtr &other) const {
 
 /** \brief Returns whether this extent intersects the other one.
  *
- * Behaviour only well specified if each sub-extent category as at most
+ * Behavior only well specified if each sub-extent category as at most
  * one element.
  */
 bool Extent::intersects(const ExtentNNPtr &other) const {
@@ -827,7 +883,7 @@ bool Extent::intersects(const ExtentNNPtr &other) const {
 
 /** \brief Returns the intersection of this extent with another one.
  *
- * Behaviour only well specified if there is one single GeographicExtent
+ * Behavior only well specified if there is one single GeographicExtent
  * in each object.
  * Returns nullptr otherwise.
  */
@@ -893,8 +949,8 @@ void Identifier::Private::setProperties(
                                                     AUTHORITY_KEY);
                 }
             } else {
-                if (auto citation =
-                        dynamic_cast<const Citation *>(pVal->get())) {
+                auto citation = dynamic_cast<const Citation *>(pVal->get());
+                if (citation) {
                     authority_ = *citation;
                 } else {
                     throw InvalidValueTypeException("Invalid value type for " +
@@ -1055,7 +1111,12 @@ const optional<std::string> &Identifier::uri() PROJ_PURE_DEFN {
 void Identifier::_exportToWKT(WKTFormatter *formatter) const {
     const bool isWKT2 = formatter->version() == WKTFormatter::Version::WKT2;
     const std::string &l_code = code();
-    const std::string &l_codeSpace = *codeSpace();
+    std::string l_codeSpace = *codeSpace();
+    std::string l_version = *version();
+    const auto &dbContext = formatter->databaseContext();
+    if (dbContext) {
+        dbContext->getAuthorityAndVersion(*codeSpace(), l_codeSpace, l_version);
+    }
     if (!l_codeSpace.empty() && !l_code.empty()) {
         if (isWKT2) {
             formatter->startNode(WKTConstants::ID, false);
@@ -1066,17 +1127,17 @@ void Identifier::_exportToWKT(WKTFormatter *formatter) const {
             } catch (const std::exception &) {
                 formatter->addQuotedString(l_code);
             }
-            if (version().has_value()) {
-                auto l_version = *(version());
-                try {
-                    (void)c_locale_stod(l_version);
+            if (!l_version.empty()) {
+                bool isDouble = false;
+                (void)c_locale_stod(l_version, isDouble);
+                if (isDouble) {
                     formatter->add(l_version);
-                } catch (const std::exception &) {
+                } else {
                     formatter->addQuotedString(l_version);
                 }
             }
             if (authority().has_value() &&
-                *(authority()->title()) != l_codeSpace) {
+                *(authority()->title()) != *codeSpace()) {
                 formatter->startNode(WKTConstants::CITATION, false);
                 formatter->addQuotedString(*(authority()->title()));
                 formatter->endNode();
@@ -1100,17 +1161,42 @@ void Identifier::_exportToWKT(WKTFormatter *formatter) const {
 
 void Identifier::_exportToJSON(JSONFormatter *formatter) const {
     const std::string &l_code = code();
-    const std::string &l_codeSpace = *codeSpace();
+    std::string l_codeSpace = *codeSpace();
+    std::string l_version = *version();
+    const auto &dbContext = formatter->databaseContext();
+    if (dbContext) {
+        dbContext->getAuthorityAndVersion(*codeSpace(), l_codeSpace, l_version);
+    }
     if (!l_codeSpace.empty() && !l_code.empty()) {
-        auto &writer = formatter->writer();
+        auto writer = formatter->writer();
         auto objContext(formatter->MakeObjectContext(nullptr, false));
-        writer.AddObjKey("authority");
-        writer.Add(l_codeSpace);
-        writer.AddObjKey("code");
+        writer->AddObjKey("authority");
+        writer->Add(l_codeSpace);
+        writer->AddObjKey("code");
         try {
-            writer.Add(std::stoi(l_code));
+            writer->Add(std::stoi(l_code));
         } catch (const std::exception &) {
-            writer.Add(l_code);
+            writer->Add(l_code);
+        }
+
+        if (!l_version.empty()) {
+            writer->AddObjKey("version");
+            bool isDouble = false;
+            (void)c_locale_stod(l_version, isDouble);
+            if (isDouble) {
+                writer->AddUnquoted(l_version.c_str());
+            } else {
+                writer->Add(l_version);
+            }
+        }
+        if (authority().has_value() &&
+            *(authority()->title()) != *codeSpace()) {
+            writer->AddObjKey("authority_citation");
+            writer->Add(*(authority()->title()));
+        }
+        if (uri().has_value()) {
+            writer->AddObjKey("uri");
+            writer->Add(*(uri()));
         }
     }
 }
@@ -1122,7 +1208,7 @@ void Identifier::_exportToJSON(JSONFormatter *formatter) const {
 //! @cond Doxygen_Suppress
 static bool isIgnoredChar(char ch) {
     return ch == ' ' || ch == '_' || ch == '-' || ch == '/' || ch == '(' ||
-           ch == ')' || ch == '.' || ch == '&';
+           ch == ')' || ch == '.' || ch == '&' || ch == ',';
 }
 //! @endcond
 
@@ -1206,14 +1292,14 @@ bool Identifier::isEquivalentName(const char *a, const char *b) noexcept {
     size_t j = 0;
     char lastValidA = 0;
     char lastValidB = 0;
-    while (a[i] != 0 && b[j] != 0) {
+    while (a[i] != 0 || b[j] != 0) {
         char aCh = a[i];
         char bCh = b[j];
-        if (aCh == ' ' && a[i + 1] == '+' && a[i + 2] == ' ') {
+        if (aCh == ' ' && a[i + 1] == '+' && a[i + 2] == ' ' && a[i + 3] != 0) {
             i += 3;
             continue;
         }
-        if (bCh == ' ' && b[j + 1] == '+' && b[j + 2] == ' ') {
+        if (bCh == ' ' && b[j + 1] == '+' && b[j + 2] == ' ' && b[j + 3] != 0) {
             j += 3;
             continue;
         }
@@ -1251,21 +1337,18 @@ bool Identifier::isEquivalentName(const char *a, const char *b) noexcept {
                 j += strlen(replacement->utf8) - 1;
             }
         }
-        if (::tolower(aCh) != ::tolower(bCh)) {
+        if ((aCh == 0 && bCh != 0) || (aCh != 0 && bCh == 0) ||
+            ::tolower(aCh) != ::tolower(bCh)) {
             return false;
         }
         lastValidA = aCh;
         lastValidB = bCh;
-        ++i;
-        ++j;
+        if (aCh != 0)
+            ++i;
+        if (bCh != 0)
+            ++j;
     }
-    while (a[i] != 0 && isIgnoredChar(a[i])) {
-        ++i;
-    }
-    while (b[j] != 0 && isIgnoredChar(b[j])) {
-        ++j;
-    }
-    return a[i] == b[j];
+    return true;
 }
 
 // ---------------------------------------------------------------------------
